import 'package:chariot/src/container/error.dart';

///Ioc Container Interface
abstract class IContainer
{
    void set<T>(Creator<T> creator);

    T get<T>({bool singleton = true});

    bool has<T>();

    void unset<T>();
}

typedef Creator<T> = T Function(IContainer container);

///Ioc Container
class Container implements IContainer
{
    final Map<Type, Creator> _objects = {};
    final Map<Type, dynamic> _created = {};

    @override
    void set<T>(Creator<T> creator)
    {
        if(!has<T>()) {
            _objects[T] = creator;
        }
    }

    @override
    T get<T>({bool singleton = true})
    {
        if(!has<T>()) {
            throw UnregisteredType("Unregistered type '$T'");
        }

        if(!singleton || _created[T] == null) {
            return _created[T] = _objects[T](this);
        }

        return _created[T];
    }

    @override
    bool has<T>()
    {
        return _objects.containsKey(T);
    }

    @override
    void unset<T>()
    {
        _objects?.remove(T);
        _created?.remove(T);
    }
}