import 'dart:io';

///Http handler type
typedef HttpHandler = void Function(HttpRequest request);

///Default http handler
void _defaultHandler(HttpRequest req)
{
    req.response..writeln('Welcome to chatiot!')..close();
}

class Server
{
    ///Http server object
    HttpServer _server;

    ///Http handler
    HttpHandler _handler = _defaultHandler;

    void setHandler(HttpHandler handler) {
        _handler = handler;
    }

    void serve({dynamic address = '127.0.0.1', int port = 8282, SecurityContext securityContext, int backlog = 0, bool shared = false}) async {
        if (_handler == null) {
            throw Exception('server handler not set');
        }

        if (securityContext == null) {
            _server = await HttpServer.bind(address, port, backlog: backlog, shared: shared);
        } else {
            _server = await HttpServer.bindSecure(address, port, securityContext, backlog: backlog, shared: shared);
        }

        _server.listen(_onData, onDone: _onDone, onError: _onError);
    }

    void stop({bool force = false})
    {
        if(_server != null) {
            _server.close(force: force);
        }
    }

    bool running()
    {
        return _server != null;
    }

    void _onData(HttpRequest request)
    {
        _handler(request);
    }

    void _onDone()
    {

    }

    void _onError(error, StackTrace stackTrace)
    {

    }
}