
import 'package:chariot/src/log/adapter/std.dart';

///Logger adapter interface
abstract class LoggerAdapter
{
    void write(String content);
}

///Logger interface
abstract class ILogger
{
    void info(String message);
    void debug(String message);
    void warn(String message);
    void error(String message);
    void fatal(String message);
}

///Log default levels
const Set<String> DefaultLogLevels = {Logger.INFO, Logger.DEBUG, Logger.WARN, Logger.ERROR, Logger.FATAL};

///Logger
class Logger extends ILogger
{
    static const INFO  = 'INFO';
    static const DEBUG = 'DEBUG';
    static const WARN  = 'WARN';
    static const ERROR = 'ERROR';
    static const FATAL = 'FATAL';

    ///Log Adapter
    final LoggerAdapter _adapter ;

    ///Log levels
    final Set<String> _levels;

    ///Init logger
    Logger({LoggerAdapter adapter, Set<String> levels}): _adapter = adapter ?? StdAdapter(), _levels = levels ?? DefaultLogLevels;

    ///log with [level]
    void log(String level, String message)
    {
        if(_levels.contains(level)) {
            _adapter.write(' $level '+message);
        }
    }

    ///`INFO` level log
    @override
    void info(String message) => log(Logger.INFO, message);

    ///`DEBUG` level log
    @override
    void debug(String message) => log(Logger.DEBUG, message);

    ///`WARN` level log
    @override
    void warn(String message) => log(Logger.WARN, message);

    ///`ERROR` level log
    @override
    void error(String message) => log(Logger.ERROR, message);

    ///`FATAL` level log
    @override
    void fatal(String message) => log(Logger.FATAL, message);
}