import 'dart:async';

import 'package:chariot/src/http/context.dart';
import 'package:chariot/src/http/request.dart';
import 'package:chariot/src/http/method.dart';

///Handler
typedef Handler = FutureOr<Request> Function(Context);

///Middleware
typedef Middleware = Handler Function(Handler);

class Action
{
    String  path;
    Handler handler;

    Action(this.path, this.handler);
}

///Route Collector
class Router
{
    ///Registered routes
    final Map<Method, Action> _routes = {};

    ///Register [method] route
    Action route(Method method, String path, Handler handler) => _routes[method] = Action(path, handler);

    ///Register `GET` method route
    Action get(String path, Handler handler) => route(Method.GET, path, handler);

    ///Register `POST` method route
    Action post(String path, Handler handler) => route(Method.POST, path, handler);

    ///Register `PUT` method route
    Action put(String path, Handler handler) => route(Method.PUT, path, handler);

    ///Register `PATCH` method route
    Action patch(String path, Handler handler) => route(Method.PATCH, path, handler);

    ///Register `DELETE` method route
    Action delete(String path, Handler handler) => route(Method.DELETE, path, handler);

    ///Register `OPTIONS` method route
    Action options(String path, Handler handler) => route(Method.OPTIONS, path, handler);

    ///Get all routes
    Map<Method, Action> get routes => _routes;

    ///Add match condition
    void where(String name, String value) {

    }
}